import { Metadata, Http } from '@zeedhi/core';
import { Modal, ModalService, Form, DialogService, LoadingService, SelectMultiple} from '@zeedhi/common';
import { TekGrid } from "@zeedhi/teknisa-components-common";
import { CustomAlertService } from '@/Services/CustomAlertService';
import { Utilities } from '@/Services/Utilities';
import { Highlight } from '@zeedhi/zd-highlight-common';
import Prism from 'prismjs';
import 'prismjs/components/prism-markup-templating.min';
import 'prismjs/components/prism-php.min';

export class DataSourceController {
    //  Referente ao componente Highlight
    public loadLanguage() {
        const { code, language } = Metadata.getInstance<Highlight>('query');
        return Prism.highlight(code, Prism.languages[language], language);
    }
    // Métodos de Dialog e Modal
    public showDeleteDialog(params: any) {
        Utilities.genericBeforeDeleteTekGrid(params, "/dataSource/remove", "datasource-grid");
    }

    private modal!: Modal;
    public closeModal() {
        this.modal.hide();
    }
    public async createDetailModal() {
        this.modal = await ModalService.createFromJson('dataSource/DetailModal/DetailModal', true);
    }
    public async createAddModal() {
        this.modal = await ModalService.createFromJson('dataSource/AddModal/AddModal', true);
    }

    protected modalData: any;
    public get modalTitle() {
        return this.modalData?.name || "Fonte de Dados";
    }
    public async openModal({ component, row }: any) {
        if (component.editing === true) return;
        this.modal?.destroy();
        if (row) {
            await this.createDetailModal();
        } else {
            await this.createAddModal();
        }
        this.modalData = row === undefined ? {joinDataSource: "N", hasDataMining: "S", willUpdateIndicators: "N", isScheduleActive: "S"} : row;
        this.modal.show();
    }
    // Métodos da query
    public extractStringsRangeFromQuery(code: string) {
        const stringRegex = /(["'])(?:(?=(\\?))\2.)*?\1/g;
        return [...code.matchAll(stringRegex)].map((codeString: any) => [codeString.index, codeString.index + codeString[0].length]);
    }

    public strReplaceNotIn(forbideenRange: any, inForbideenFunc: any, notInForbideenFunc: any) {
        return (...params: any) => {
            const commentStart = params[params.length - 2];
            const containingString = forbideenRange.find(([start, end]: any) => commentStart > start && commentStart < end);
            if (containingString) {
                return inForbideenFunc(params, containingString);
            }
            return notInForbideenFunc(params);
        };
    }

    public clearComments(code: string): string {
        if (code === "") {
            return "";
        }
        const commentRegex = /(--.*)|(((\/\*)+?[\w\W]+?(\*\/)+))/g;
        const stringRanges = this.extractStringsRangeFromQuery(code);
        return code.replace(commentRegex, this.strReplaceNotIn(stringRanges, (params: any, [start, end]: any) => {
            const comment = params[0];
            const onString = comment.slice(0, end - start);
            const notOnString = comment.slice(end - start - 1, comment.length);
            return onString + this.clearComments(notOnString);
        }, () => ""));
    }

    public parseQueryCode(code: string) {
        const doubleWhiteSpaceRegex = /\s+/g;
        const commentless = this.clearComments(code);
        const stringRanges = this.extractStringsRangeFromQuery(commentless);
        const parsedCode = commentless.replace(doubleWhiteSpaceRegex, this.strReplaceNotIn(stringRanges, (params: any) => params[0], () => " "));
        return parsedCode.trim();
    }
    //  Metodos do Form
    public fillDataSourceForm({ row }: any) {
        const values = row || this.modalData;
        const form = Metadata.getInstance<Form>("dataSourceForm");
        const query = Metadata.getInstance<Highlight>('query');
        const accessProfiles = form.getChildInstance<SelectMultiple>("accessProfiles");
        form.value = values;
        query.code = values.query_view || values.query || "";
        accessProfiles.setValue(values.accessProfiles || []);
    }

    public async saveDatasourceForm() {
        const form = Metadata.getInstance<Form>("dataSourceForm");
        const query = Metadata.getInstance<Highlight>('query');
        const values = form.value;
        const connection = values.connection;
        const queryCode = query.code ? this.parseQueryCode(query.code) : "";
        if (!form.validate() || !queryCode.trim()) {
            if (!queryCode.trim()) {
                CustomAlertService.errorAlert("O campo de código é obrigatório e deve ser preenchido");
            }
            return;
        }
        const data = {
            ...values,
            changed: true,
            columns: this.modalData.columns || [],
            connection: connection?._id,
            connectionName: connection?.name,
            room: connection?.room,
            createdBy: this.modalData.userId,
            importStatus: this.modalData.importStatus || "",
            originDataSources: this.modalData.originDataSources || "",
            query: queryCode,
            query_view: query.code || "",
            relations: this.modalData.relations || [],
            _queryChanged: this.modalData._id ? !(this.modalData.query === queryCode) : true
        };
        try {
            LoadingService.show();
            const res = await Http.post("/dataSource/save", {
                requestType: "DataSet",
                disableLoader: false,
                dataset: [this.modalData._id ? data : {
                    ...data,
                    __createdLocal: true,
                    __is_new: true,
                    createdBy: this.user.userId
                }]
            });
            LoadingService.hide();
            CustomAlertService.saveSuccessAlert();
            if (res.data.messages?.length > 0) {
                const message = res.data.messages[0].message;
                DialogService.show({
                  name: 'warning-dialog',
                  type: 'error',
                  title: ' ',
                  text: message
                });
            }
            this.reloadGrid();
            this.closeModal();
        } catch (err) {
            LoadingService.hide();
            CustomAlertService.errorAlert("Ocorreu um erro.");
            console.error(err);
        }
    }

    protected user: any;
    public async updateLoggedUserInfo() {
        Http.post("/getLoggedUser", {nrorg: null, nrorgPrincipal: null}).then((res) => {
            const login = res.data;
            this.user = {
                userId: login.userId,
                username: login.username
            };
        });
    }

    public handleConnectionResponse({ component }: any) {
        const info = component?.currentRow;
        if (info?.WALLET_NRORG === undefined) {
            return;
        }
        const {data, WALLET_NRORG, pagination} = info;
        component.data = data;
        component.total = pagination.total;
        component.limit = pagination.limit;
        component.page = pagination.page;
        component.currentRow = data;
    }
    public reloadGrid() {
        const grid = Metadata.getInstance<TekGrid>("datasource-grid");
        grid.reload();
    }
    public async saveInlineEdit(params: any) {
        Utilities.genericBeforeSaveTekGrid(params, "dataSource/save", "datasource-grid");
    }

    public adjustFilter({ component }: any) {
        const datasource = component.datasource;
        const filter = datasource.dynamicFilter;
        Object.entries(filter).forEach(([k, filters]: any) => {
            filter[k] = filters.filter(({value}: any) => !(value instanceof Array && value.length === 0));
            if (filter[k].length === 0) {
                delete filter[k];
            }
        });
    }
}
