import { Http, Metadata } from '@zeedhi/core';
import { Modal, ModalService, Select, DialogService, Form} from '@zeedhi/common';
import { Highlight } from '@zeedhi/zd-highlight-common';
import { TekGrid, TekGridColumn, TekRestDatasource } from '@zeedhi/teknisa-components-common';
import { CustomAlertService } from '@/Services/CustomAlertService';

import Prism from 'prismjs';
import 'prismjs/components/prism-markup-templating.min';
import 'prismjs/components/prism-php.min';

Metadata.get('metadata/queryRunner.json');

export class QueryRunnerController {
    public bindModal!: Modal;

    public query: string = '';
    public binds: any = {};
    public currentBinds: any = [];

    public closeBindModal() {
        this.bindModal.hide();
    }
    public async openBindModal() {
        this.bindModal?.destroy();
        this.bindModal = await ModalService.createFromJson('queryRunner/bindModal', true);
        const container = this.bindModal.children.find(({ component }) => component === "ZdForm");
        if (container) {
            container.children = this.currentBinds.map(({ value }: any) => this.generateListItem(value, this.binds[value])).flat();
        }
        this.bindModal.show();
    }
    public executeAfterDatasourceDialog() {
        DialogService.show({
          name: 'executeAfterDatasourceDialog',
          type: 'info',
          title: ' ',
          text: 'Deseja executar query?',
          buttons: [
            { name: 'não', component: 'ZdButton', label: 'NÃO', outline: true, events: { click: () => DialogService.hide(), }, },
            { name: 'sim', component: 'ZdButton', label: 'SIM', events: { click: () => { DialogService.hide(); this.beforeRunQuery(); }}, },
          ],
        });
    }

    public loadLanguage() {
        const { code, language } = Metadata.getInstance<Highlight>('query');
        return Prism.highlight(code, Prism.languages[language], language);
    }

    public accessProfiles: any = [];
    public mainCardMounted() {
        const datasource = Metadata.getInstance<Select>('datasource').datasource as TekRestDatasource;
        Http.post("/getLoggedUser", {nrorg: null, nrorgPrincipal: null}).then((res) => {
            const login = res.data;
            this.accessProfiles = login.accessProfiles;
            datasource.dynamicFilter = {
                "accessProfiles": this.accessProfiles.map((value: any) => ({
                    "operation": "CONTAINS",
                    "relation": "OR",
                    value
                }))
            };
            datasource.get();
        });
    }

    public beforeRunQuery() {
        const form = Metadata.getInstance<Form>('queryRunnerForm');
        if (!form.validate() || !this.query.trim()) {
            if (!this.query.trim()) {
              CustomAlertService.errorAlert("O campo de código é obrigatório e deve ser preenchido");
            }
            return;
        }
        this.currentBinds = this.parseQueryBinds(this.query).map((value: any, id: any) => ({id, value}));
        if (this.currentBinds.length > 0) {
            this.openBindModal();
            return;
        }
        this.runQuery();
    }
    public async runQuery(query: string = this.query) {
        const connection = Metadata.getInstance<Select>('conectionSelect');
        const grid = Metadata.getInstance<TekGrid>('queryRunnerGrid');
        const uri = '/runQuery';
        const requestBody = {requestType: "Row", row: {}};
        for (let i = 0; i < connection.datasource.data.length; i += 1) {
          if (connection.datasource.data[i]._id === connection.value) {
              const data = connection.datasource.data[i];
              requestBody.row = {changed: true,
              connectionDbUrl: data.dbUrl,
              connectionId: data._id,
              connectionName: data.name,
              connectionPassword: data.password,
              connectionPasswordSsl: data.passwordSsl,
              connectionUsername: data.username,
              driverClass: data.driverClass,
              oracleWalletUse: data.oracleWalletUse,
              query,
              room: data.room
              };
          }
        }
        grid.datasource.loading = true;
        await Http.post(uri, requestBody).then((queryResult: any) => {
           const result = queryResult.data;
           if (result.error) {
               DialogService.show({
                    name: 'error-dialog',
                    type: 'error',
                    title: 'Ocorreu um erro',
                    text: result.error
                });
                grid.datasource.loading = false;
                return;
           }
           const columns = result.dataset.queryResult;
           const newColumns = columns.length > 0 ? Object.keys(columns[0]) : [];
           const tekgridcolumns = newColumns.map((item: any) => (new TekGridColumn({name: item, label: item, sortable: false}, grid)));
           grid.columns = tekgridcolumns;
           grid.datasource.updateData(columns);
           grid.datasource.loading = false;
           grid.reload();
        });
    }

    public handleDatasourceSelection({ value }: any) {
        if (value) {
            const query = Metadata.getInstance<Highlight>('query');
            const connection = Metadata.getInstance<Select>('conectionSelect');
            query.code = value.query_view || value.query || "";
            connection.setValue(value.connection || "");
            this.executeAfterDatasourceDialog();
        }
    }
    public generateListItem(name: string, value: any = "") {
      const label = {
        "name": name + "_label",
        "component": "ZdText",
        "text": name,
        "readonly": true,
        "showLabel": false,
        "showHelper": false,
        "cssClass": "zd-mt-2 zd-col-4",
        "cssStyle": "font-weight: bold;",
        "disabled": true
      };
      const component = this.specialBinds[name] ? this.specialBinds[name](name, value) : {
        "name": name,
        "component": "ZdTextInput",
        "value": value,
        "showLabel": false,
        "cssClass": "zd-mt-2 zd-col-8",
        "validations": { "required": {} }
      };
      return {
          "name": name + "_row",
          "component": "ZdRow",
          "cssClass": "zd-align-center",
          "grid": { "cols": 12 },
          "children": [label, component]
      };
    }

    public specialBindsDef = [
      {
        field: [":START_DATE", ":END_DATE"],
        keyword: "Período",
        revertValues: (value: any) => [...value],
        component: (name: string, value: any) => ({
            "name": name,
            "component": "ZdDateRange",
            "value": value,
            "showLabel": false,
            "dateFormat": "DD/MM/YYYY",
            "inputFormat": "DD/MM/YYYY",
            "displayFormat": "DD/MM/YYYY",
            "splitter": " - ",
            "orderedDates": true,
            "cssClass": "zd-mt-2 zd-col-8",
            "validations": { "required": {} }
        })
      }
    ];

    public get specialBinds() {
      return this.specialBindsDef.reduce((obj: any, {field, keyword, component}: any) => {
        obj[keyword || field] = component;
        return obj;
      }, {});
    }
    public handleSpecialBinds(keyBinds: any = []) {
      this.specialBindsDef.filter(({keyword}) => keyword).forEach(({field, keyword}) => {
        if (Array.isArray(field)) {
          if (field.reduce((isIn, name) => isIn && keyBinds.includes(name), true)) {
            keyBinds = keyBinds.filter((value: string) => !field.includes(value));
            keyBinds.push(keyword);
          }
        } else if (keyBinds.includes(field)) {
          keyBinds = keyBinds.filter((value: string) => value !== field);
          keyBinds.push(keyword);
        }
      });
      return keyBinds;
    }
    public revertSpecialBinds(binds: any = {}) {
        Object.keys(binds).forEach((name) => {
            const bindDef = this.specialBindsDef.find(({ keyword }: any) => keyword === name);
            if (bindDef) {
                const value = binds[name];
                const revertedValue = bindDef.revertValues(value);
                if (Array.isArray(bindDef.field)) {
                    bindDef.field.forEach((field: string, i: number) => { binds[field] = revertedValue[i]; });
                } else {
                    binds[bindDef.field] = revertedValue;
                }
                delete binds[name];
            }
        });
        return binds;
    }
    public parseQueryBinds(query: string, binds: any = null) {
        const stringRegex = /(["'])(?:(?=(\\?))\2.)*?\1/g;
        const bindsRegex = /:(\w)+/g;
        const stringRanges = [...query.matchAll(stringRegex)].map((codeString: any) => [codeString.index, codeString.index + codeString[0].length]);
        const keyBinds: any = [];
        const result = query.replace(bindsRegex, (...params: any) => {
            const bindStart = params[params.length - 2];
            const containingString = stringRanges.find(([start, end]: any) => bindStart > start && bindStart < end);
            if (!containingString) {
                if (binds && binds[params[0]]) {
                    return "'" + binds[params[0]] + "'";
                }
                keyBinds.push(params[0]);
            }
            return params[0];
        });
        return binds ? result : this.handleSpecialBinds([...new Set(keyBinds)]);
    }

    public saveBinds() {
        const form = Metadata.getInstance<Form>('detailForm');
        if (!form.validate()) {
            return;
        }
        const bindsValues = form.childrenInstances?.map((row: any) => row.childrenInstances[1]).reduce((obj: any, input: any) => { obj[input.name] = input.value; return obj; }, {});
        Object.keys(bindsValues).forEach((name: string) => { this.binds[name] = bindsValues[name]; });
        const parsedQuery = this.parseQueryBinds(this.query, this.revertSpecialBinds(bindsValues));
        this.closeBindModal();
        this.runQuery(parsedQuery);
    }
}
